#define VERSION "0.1"
#define ID "$Id: main.c 2916 2013-04-14 17:42:03Z kaklik $"

#include <main.h>
#include <math.h>
#include <string.h>

#include "SHT25.h"
#include "MPL3115.h"
#include "HMC5883L.h"
#include ".\common\dbloader.h"


unsigned int16 timer0_overflow_count;
float anemo=0;
unsigned int8 rain;

#int_TIMER1
void  TIMER1_isr(void) 
{
// 32.768 kHz krystal pro timer1 osciltor
  anemo = ((timer0_overflow_count * 0xFF) + get_timer0())/(0xFFFF/32768.0); // pocet pulzu za 1s

  timer0_overflow_count=0; //nulovani
  set_timer0(0);
  set_timer1(0);
  output_toggle(PIN_E0);
}

#int_TIMER0 //pro preteceni tae plz od anemometru (RA4)
void  TIMER0_isr(void) 
{
  timer0_overflow_count++;
}

#INT_EXT
void EXT_isr()  //interrup from rain sensor clip. 
{
  rain++;
//  if (input(PIN_B0)) ext_int_edge( H_TO_L );     osetreni pro pripad, ze by bylo treba cist obe hrany impulzu
//  if (!input(PIN_B0)) ext_int_edge( L_TO_H );
}

float wind_direction(void) //vypocet azimutu smeru vetru
{
float b;
  hmc5883l_read_data();       // reads magnetometer data to global variables
 
  b = atan2((float)compass.y,(float)compass.x); // vypocet azimutu z kartezskych souradnic
  b = (b/3.141596)*180;  // prevod na stupne
  b += 180;
  return b;  
}


void welcome(void)    // uvodni zprava
{
   printf("\r\n\r\n# Meteorologicka stanice %s (C) 2013 www.mlab.cz \r\n",VERSION); 
   printf("\r\n %s \r\n",ID);// Welcome message
   printf("# ver poradi ");
   printf("altimet_t[C] altimet_a[m] altimet_p[Pa] ");
   printf("sht_t[C] sht_h[%%] sht_config Wind_direction Anemo[pls/s]check\r\n\r\n");
}

void main()
{
   setup_adc_ports(NO_ANALOGS|VSS_VDD);
   setup_adc(ADC_CLOCK_DIV_2);
   setup_spi(SPI_SS_DISABLED);
   setup_timer_0(RTCC_EXT_L_TO_H|RTCC_DIV_1);
   setup_timer_1(T1_EXTERNAL|T1_DIV_BY_1|T1_CLK_OUT);
   setup_timer_2(T2_DISABLED,0,1);
   setup_ccp1(CCP_OFF);
   setup_comparator(NC_NC_NC_NC);// This device COMP currently not supported by the PICWizard


float altimet_t;  //teplota z MPL3115
float altimet_p;  //tlak z MPL3115
float altimet_a;  //vyska z MPL3115
float sht25_t;    //teplota z SHT25
float sht25_h;    //relativni vlhkost z SHT25
float smer_v;     //smer vetru
 
unsigned int8 sht_config;

//nastaven peruen pro anemometr
enable_interrupts(INT_TIMER1);   
enable_interrupts(INT_TIMER0);
enable_interrupts(INT_EXT);
enable_interrupts(GLOBAL);
// vynulovani promenych pro anemometr
set_timer0(0);
set_timer1(0);
timer0_overflow_count=0;
rain=0;

welcome();

// Init the HMC5883L.  Set Mode register for
// continuous measurements.
hmc5883l_write_reg(HMC5883L_CFG_A_REG, 0x18);      // no average, maximal update range
hmc5883l_write_reg(HMC5883L_CFG_B_REG, 0xE0);      // maximal range
hmc5883l_write_reg(HMC5883L_MODE_REG, 0x00);

//nastaveni SHT25
SHT25_soft_reset();
sht_config = SHT25_RH12_T14 | SHT25_HEATER_OFF; //vypnuti topeni v SHT25
SHT25_setup(sht_config);


   while(TRUE)
   {
unsigned int32 i=0;

     mpl3115_setP(); //nastaveni pro tlak a teplotu
     delay_ms (500);
     altimet_t=mpl3115_T();
     altimet_p=mpl3115_P();
   
     mpl3115_setA(); //nastaveni pro vysku a teplotu
     delay_ms (500);
     altimet_a = mpl3115_A();

    if (i<10000) sht_config = SHT25_RH12_T14 | SHT25_HEATER_OFF; // loop alters on chip heater on and off to check correct function
    else
    {
      sht_config = SHT25_RH12_T14 | SHT25_HEATER_ON;
      if (i > 10050) i = 0;
    }

   
     sht25_t = SHT25_get_temp(); //mereni hodnot z SHT25
     sht25_h = SHT25_get_hum();
     SHT25_setup(sht_config);
  
     smer_v = wind_direction(); //vrati azimut aktualniho smeru vetru

     printf(" %6.2f %7.2f %9.2f %6.2f %6.2f %X %5.2f %6.1f %u \n\r", altimet_t, altimet_a, altimet_p, sht25_t, sht25_h, sht_config, smer_v, anemo, rain); 
 
     delay_ms(1000);
     rain=0;
     i++;
  }
}

